@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: EUPL-1.2
+# SPDX-License-Identifier: MIT
 # SPDX-FileCopyrightText: 2021 Quansight, LLC
 # SPDX-FileCopyrightText: 2021 Filipe Laíns <lains@riseup.net>
 
@@ -46,15 +46,14 @@ from mesonpy._compat import Collection, Iterator, Mapping, Path
 
 
 if typing.TYPE_CHECKING:  # pragma: no cover
-    import pep621 as _pep621  # noqa: F401
+    import pyproject_metadata  # noqa: F401
     import wheel.wheelfile  # noqa: F401
 
 
-__version__ = '0.5.0'
+__version__ = '0.6.0'
 
 
 class _depstr:
-    ninja = 'ninja >= 1.10.0'
     patchelf = 'patchelf >= 0.11.0'
     wheel = 'wheel >= 0.36.0'  # noqa: F811
 
@@ -185,11 +184,11 @@ class _WheelBuilder():
         except ModuleNotFoundError:
             return False
 
-    def _is_elf(self, file: str) -> bool:
+    def _is_elf(self, file: Union[str, pathlib.Path]) -> bool:
         with open(file, 'rb') as f:
             return f.read(4) == b'\x7fELF'
 
-    def _warn_unsure_platlib(self, origin: str, destination: pathlib.Path) -> None:
+    def _warn_unsure_platlib(self, origin: pathlib.Path, destination: pathlib.Path) -> None:
         if self._is_elf(origin):
             return
         warnings.warn(
@@ -198,7 +197,7 @@ class _WheelBuilder():
             stacklevel=2,
         )
 
-    def _map_from_heuristics(self, origin: str, destination: pathlib.Path) -> Optional[Tuple[str, pathlib.Path]]:
+    def _map_from_heuristics(self, origin: pathlib.Path, destination: pathlib.Path) -> Optional[Tuple[str, pathlib.Path]]:
         """Extracts scheme and relative destination with heuristics based on the
         origin file and the Meson destination path.
         """
@@ -208,13 +207,14 @@ class _WheelBuilder():
         sys_paths = sysconfig.get_paths(vars=sys_vars)
         # Debian dist-packages
         if self._debian_python:
-            search_path = destination
+            search_path = origin
             while search_path != search_path.parent:
                 search_path = search_path.parent
                 if search_path.name == 'dist-packages' and search_path.parent.parent.name == 'lib':
-                    calculated_path = destination.relative_to(search_path)
+                    calculated_path = origin.relative_to(search_path)
                     warnings.warn(f'File matched Debian heuristic ({calculated_path}): {origin} ({destination})')
-                    self._warn_unsure_platlib(origin, destination)
+                    if not destination.root == '{moduledir_shared}':
+                        self._warn_unsure_platlib(origin, destination)
                     return 'platlib', calculated_path
         # purelib or platlib -- go to wheel root
         for scheme in ('purelib', 'platlib'):
@@ -227,10 +227,10 @@ class _WheelBuilder():
                 continue
             # {moduledir_shared} is currently handled in heuristics due to a Meson bug,
             # but we know that files that go there are supposed to go to platlib
-            if sys_paths['purelib'] == sys_paths['platlib'] and not origin.startswith('{moduledir_shared}'):
+            if sys_paths['purelib'] == sys_paths['platlib'] and not destination.root == '{moduledir_shared}':
                 self._warn_unsure_platlib(origin, destination)
             return 'platlib', wheel_path
-        return None
+        return None  # no match was found
 
     def _map_from_scheme_map(self, destination: str) -> Optional[Tuple[str, pathlib.Path]]:
         """Extracts scheme and relative destination from Meson paths.
@@ -245,7 +245,7 @@ class _WheelBuilder():
             if destination.startswith(placeholder):
                 relative_destination = pathlib.Path(destination).relative_to(placeholder)
                 return scheme, relative_destination
-        return None
+        return None  # no match was found
 
     def _map_to_wheel(
         self,
@@ -263,7 +263,10 @@ class _WheelBuilder():
                     # using scheme map
                     self._map_from_scheme_map(meson_destination)
                     # using heuristics
-                    or self._map_from_heuristics(copy_files[file], pathlib.Path(meson_destination))
+                    or self._map_from_heuristics(
+                        pathlib.Path(copy_files[file]),
+                        pathlib.Path(meson_destination),
+                    )
                 )
                 if install_details:
                     scheme, destination = install_details
@@ -354,7 +357,7 @@ class Project():
     _ALLOWED_DYNAMIC_FIELDS: ClassVar[List[str]] = [
         'version',
     ]
-    _metadata: Optional[_pep621.StandardMetadata]
+    _metadata: Optional[pyproject_metadata.StandardMetadata]
 
     def __init__(
         self,
@@ -373,11 +376,11 @@ class Project():
         self._pep621 = 'project' in self._config
         if self.pep621:
             try:
-                import pep621  # noqa: F811
+                import pyproject_metadata  # noqa: F811
             except ModuleNotFoundError:  # pragma: no cover
                 self._metadata = None
             else:
-                self._metadata = pep621.StandardMetadata.from_pyproject(self._config, self._source_dir)
+                self._metadata = pyproject_metadata.StandardMetadata.from_pyproject(self._config, self._source_dir)
         else:
             print(
                 '{yellow}{bold}! Using Meson to generate the project metadata '
@@ -385,16 +388,8 @@ class Project():
             )
             self._metadata = None
 
-        # check for unsupported dynamic fields
         if self._metadata:
-            unsupported_dynamic = {
-                key for key in self._metadata.dynamic
-                if key not in self._ALLOWED_DYNAMIC_FIELDS
-            }
-            if unsupported_dynamic:
-                raise MesonBuilderError('Unsupported dynamic fields: {}'.format(
-                    ', '.join(unsupported_dynamic)),
-                )
+            self._validate_metadata()
 
         # make sure the build dir exists
         self._build_dir.mkdir(exist_ok=True)
@@ -425,9 +420,6 @@ class Project():
         # set version if dynamic (this fetches it from Meson)
         if self._metadata and 'version' in self._metadata.dynamic:
             self._metadata.version = self.version
-            # version is no longer dynamic
-            # XXX: Should this be automatically handled by pep621/pyproject-metadata?
-            self._metadata.dynamic.remove('version')
 
     def _proc(self, *args: str) -> None:
         print('{cyan}{bold}+ {}{reset}'.format(' '.join(args), **_STYLES))
@@ -462,6 +454,28 @@ class Project():
             else:
                 raise
 
+    def _validate_metadata(self) -> None:
+        assert self._metadata
+
+        # check for unsupported dynamic fields
+        unsupported_dynamic = {
+            key for key in self._metadata.dynamic
+            if key not in self._ALLOWED_DYNAMIC_FIELDS
+        }
+        if unsupported_dynamic:
+            raise MesonBuilderError('Unsupported dynamic fields: {}'.format(
+                ', '.join(unsupported_dynamic)),
+            )
+
+        # check if we are running on an unsupported interpreter
+        if self._metadata.requires_python:
+            self._metadata.requires_python.prereleases = True
+            if platform.python_version().rstrip('+') not in self._metadata.requires_python:
+                raise MesonBuilderError(
+                    f'Unsupported Python version `{platform.python_version()}`, '
+                    f'expected `{self._metadata.requires_python}`'
+                )
+
     @functools.lru_cache(maxsize=None)
     def build(self) -> None:
         self._meson('compile')
@@ -551,8 +565,8 @@ class Project():
                 Name: {self.name}
                 Version: {self.version}
             ''').strip().encode()
-        # re-import pep621 to raise ModuleNotFoundError if it is really missing
-        import pep621  # noqa: F401, F811
+        # re-import pyproject_metadata to raise ModuleNotFoundError if it is really missing
+        import pyproject_metadata  # noqa: F401, F811
         assert self._metadata
         # use self.version as the version may be dynamic -- fetched from Meson
         core_metadata = self._metadata.as_rfc822()
@@ -801,7 +815,7 @@ def build_sdist(
 def get_requires_for_build_wheel(
     config_settings: Optional[Dict[str, str]] = None,
 ) -> List[str]:
-    dependencies = [_depstr.wheel, _depstr.ninja]
+    dependencies = [_depstr.wheel]
     with _project(config_settings) as project:
         if not project.is_pure and platform.system() == 'Linux':
             # we may need patchelf
